#!/usr/bin/haserl
<? 
. /usr/lib/webif/webif.sh

dpkg -l phidget22networkserver | grep ^ii 2>&1 1>/dev/null && installed="true" || installed="false"
p22nsconfig=/etc/phidgets/phidget22networkserver.pc
[ -f $p22nsconfig ] && phidget22networkserverconfiggood="true"


header_js()
{
	cat <<EOF
<style type="text/css">
<!--
#mytable table {
	margin-left: 1em;
	margin-right: 1em;
	margin-bottom: 1.5em;
	text-align: left;
	font-size: 0.8em;
	border-style: none;
	border-spacing: 0;
	border: thin solid black;
}
#mytable td {
	padding-left: 0.5em;
	padding-right: 1.0em;
}
-->
</style>

<script type="text/javascript">
/* <![CDATA[ */

function saved_dict_selected(i)
{
	document.getElementById("deletedict").disabled = false;
	document.getElementById("enabledict").disabled = false;
	document.getElementById("disabledict").disabled = false;
}

/* ]]> */
</script>
EOF
}

if [ "$installed" = "true" ] && [ "$phidget22networkserverconfiggood" = "true" ]; then

	ls /etc/rc4.d/*phidget22networkserver 1> /dev/null 2>&1 && enabled="true" || enabled="false"
	_p22ns_dictdir="$(pconf_get $p22nsconfig phidget.feature.dictionary.directory /etc/phidgets/dictionary.d)"
	[ -d "$_p22ns_dictdir" ] && _p22ns_dictdir_exists="true" || _p22ns_dictdir_exists="false"

	if ! empty "$FORM_submit"; then

		if equal "$FORM_startstop" "Start"; then
			/etc/init.d/phidget22networkserver start 1>/dev/null 2>&1
		elif equal "$FORM_startstop" "Stop"; then
			/etc/init.d/phidget22networkserver stopwait 1>/dev/null 2>&1
		elif ! empty "$FORM_newdict"; then
			validate <<EOF
int|newserial|Serial Number|required min=1000 max=1000000 cfg|$FORM_newserial
EOF

			equal "$?" 0 && {
				# add dir if not exists
				[ "$_p22ns_dictdir_exists" = "false" ] && mkdir -p "$_p22ns_dictdir"

				# stop if file exists
				[ -f "$_p22ns_dictdir/$FORM_newserial.dpc" ] && {
					ERROR="Dictionary with this serial number already exists"
				} || {
					cat > "$_p22ns_dictdir/$FORM_newserial.dpc" <<EOF
dictionary {
	enabled: true
	sn: $FORM_newserial
	label: '$FORM_newlabel'
}
EOF
					MESSAGE="Dictionary Added."
				}
			}
			
		# enabling a dictionary
		elif ! empty "$FORM_enabledict"; then

			sn=$FORM_dict_select
			pconf_set $_p22ns_dictdir/$sn.dpc dictionary.enabled B 1

			MESSAGE="Dictionary enabled."

		# disabling a network
		elif ! empty "$FORM_disabledict"; then

			sn=$FORM_dict_select
			pconf_set $_p22ns_dictdir/$sn.dpc dictionary.enabled B 0

			MESSAGE="Dictionary disabled. This dictionary will not be created."

		# deleting a network
		elif ! empty "$FORM_deletedict"; then

			sn=$FORM_dict_select
			rm -f $_p22ns_dictdir/$sn.dpc 1> /dev/null 2>&1

			MESSAGE="Dictionary deleted."
		else
			validate <<EOF
string|phidgetserver_servername|Server Name|min=1 max=63 cfg|$FORM_phidgetserver_servername
int|phidgetserver_ipv4_port|Port|required min=1 max=65535 cfg|$FORM_phidgetserver_ipv4_port
int|webserver_port|Port|required min=1 max=65535 cfg|$FORM_webserver_port
password|phidgetserver_password|Password|min=1 max=63 cfg|$FORM_phidgetserver_password
EOF

			equal "$?" 0 && {
				[ "$enabled" = "false" ] && [ "$FORM_enabled" = "true" ] && update-rc.d phidget22networkserver defaults 2>&1 1>/dev/null
				[ "$enabled" = "true" ] && [ "$FORM_enabled" = "false" ] && update-rc.d phidget22networkserver remove 2>&1 1>/dev/nul
				ls /etc/rc4.d/*phidget22networkserver 1> /dev/null 2>&1 && enabled="true" || enabled="false"

				pconf_set $p22nsconfig phidget.enabled B $FORM_main_phidgetserver_enable
				pconf_set $p22nsconfig phidget.network.publish.enabled B $FORM_publishenabled
				pconf_set $p22nsconfig phidget.network.ipv4.port I $FORM_phidgetserver_ipv4_port

				if empty "$FORM_phidgetserver_servername"; then
					pconf_delete $p22nsconfig phidget.network.publish.name
				else
					pconf_set $p22nsconfig phidget.network.publish.name S $FORM_phidgetserver_servername
				fi

				if empty "$FORM_phidgetserver_password"; then
					pconf_delete $p22nsconfig phidget.auth.password
				else
					pconf_set $p22nsconfig phidget.auth.password S $FORM_phidgetserver_password
				fi
				
				pconf_set $p22nsconfig phidget.www.enabled B $FORM_main_webserver_enable
				pconf_set $p22nsconfig phidget.www.network.ipv4.port I $FORM_webserver_port
				
				pconf_set $p22nsconfig phidget.feature.dictionary.enabled B $FORM_dictenabled
				pconf_set $p22nsconfig phidget.feature.control.enabled B $FORM_controldictenabled
			}
		fi
	fi

	_p22ns_enabled=$(pconf_get $p22nsconfig phidget.enabled 1)
	[ "$_p22ns_enabled" = "1" ] && _p22ns_enabled="true" || _p22ns_enabled="false"
	_p22ns_publishenabled=$(pconf_get $p22nsconfig phidget.network.publish.enabled 1)
	[ "$_p22ns_publishenabled" = "1" ] && _p22ns_publishenabled="true" || _p22ns_publishenabled="false"
	_p22ns_port=$(pconf_get $p22nsconfig phidget.network.ipv4.port 5661)
	_p22ns_www_enabled=$(pconf_get $p22nsconfig phidget.www.enabled 1)
	[ "$_p22ns_www_enabled" = "1" ] && _p22ns_www_enabled="true" || _p22ns_www_enabled="false"
	_p22ns_www_port=$(pconf_get $p22nsconfig phidget.www.network.ipv4.port 8080)
	_p22ns_servername="$(pconf_get $p22nsconfig phidget.network.publish.name)"
	_p22ns_password="$(pconf_get $p22nsconfig phidget.auth.password)"
	_p22ns_dictenabled=$(pconf_get $p22nsconfig phidget.feature.dictionary.enabled 1)
	[ "$_p22ns_dictenabled" = "1" ] && _p22ns_dictenabled="true" || _p22ns_dictenabled="false"
	_p22ns_controldictenabled=$(pconf_get $p22nsconfig phidget.feature.control.enabled 1)
	[ "$_p22ns_controldictenabled" = "1" ] && _p22ns_controldictenabled="true" || _p22ns_controldictenabled="false"


	# Try to find addr that user has typed into brower, other wise fall back on eth0 addr, and finally on wlan0 addr
	_host="${HTTP_REFERER#http://}"
	_host="${_host%%/*}"
	if empty $_host; then
		_host="$(hostname -I | awk '{print $1}')"
	fi

	hostname=$( hostname )
	
	FORM_enabled="$enabled"
	FORM_main_phidgetserver_enable="$_p22ns_enabled"
	FORM_publishenabled="$_p22ns_publishenabled"
	FORM_phidgetserver_servername="$_p22ns_servername"
	FORM_phidgetserver_password="$_p22ns_password"
	FORM_phidgetserver_ipv4_port="$_p22ns_port"
	FORM_main_webserver_enable="$_p22ns_www_enabled"
	FORM_webserver_port="$_p22ns_www_port"
	FORM_dictenabled="$_p22ns_dictenabled"
	FORM_controldictenabled="$_p22ns_controldictenabled"
fi

if [ "$installed" = "true" ] && [ "$phidget22networkserverconfiggood" = "true" ]; then
	header_inject_head=$( header_js )

	header "Phidgets" "phidget22 Network Server" "Phidget22 Network Server" "$SCRIPT_NAME"
else
	header "Phidgets" "phidget22 Network Server" "Phidget22 Network Server"
fi


if [ "$installed" = "true" ] && [ "$phidget22networkserverconfiggood" = "true" ]; then
?>
<script type="text/javascript">
/* <![CDATA[ */
function clearServerID()
{
  if(document.getElementById("phidgetserver_servername").value == "<? echo -n "$hostname" ?>")
  {
	document.getElementById("phidgetserver_servername").style.color = "black"
	document.getElementById("phidgetserver_servername").value = ""
  }
}

function resetServerID()
{
  if(document.getElementById("phidgetserver_servername").value == "")
  {
	document.getElementById("phidgetserver_servername").value = "<? echo -n "$hostname" ?>"
	document.getElementById("phidgetserver_servername").style.color = "gray"
  }
}
/* ]]> */
</script>
<?

if empty "$FORM_phidgetserver_servername"; then
	phidgetserver_servername_form="<input id=\"phidgetserver_servername\" type=\"text\" name=\"phidgetserver_servername\" value=\"$hostname\" style=\"color:gray\" onfocus=\"clearServerID()\" onclick=\"clearServerID()\" onblur=\"resetServerID()\"/>"
else
	phidgetserver_servername_form="<input id=\"phidgetserver_servername\" type=\"text\" name=\"phidgetserver_servername\" value=\"$FORM_phidgetserver_servername\" onfocus=\"clearServerID()\" onclick=\"clearServerID()\" onblur=\"resetServerID()\"/>"
fi


running="Stopped"
ps auxww | grep -v grep | grep -q "\/usr\/bin\/phidget22networkserver " && running="Running"

if equal "$running" "Stopped"; then
	startstop="Network server is <span style=\"color:red\">stopped</span>... <input type=\"submit\" value=\"Start\" name=\"startstop\" />"
else
	startstop="Network server is <span style=\"color:green\">running</span>... <input type=\"submit\" value=\"Stop\" name=\"startstop\" />"
fi

display_form <<EOF
start_form|Phidget Network Server
field|
radio|enabled|$FORM_enabled|true|Enabled
radio|enabled|$FORM_enabled|false|Disabled
field|<br />
field|
string|$startstop
field|<br />
field|<h4>Phidget Server Settings</h4>
field|<br />
field|
radio|main_phidgetserver_enable|$FORM_main_phidgetserver_enable|true|Enabled
radio|main_phidgetserver_enable|$FORM_main_phidgetserver_enable|false|Disabled
field|<br>
field|Publish
radio|publishenabled|$FORM_publishenabled|true|Enabled
radio|publishenabled|$FORM_publishenabled|false|Disabled
field|Server ID
string|$phidgetserver_servername_form
field|Port
text|phidgetserver_ipv4_port|$FORM_phidgetserver_ipv4_port
field|Password
text|phidgetserver_password|$FORM_phidgetserver_password
field|<br />
field|<h4>Webserver Settings</h4>
field|<br />
field|
radio|main_webserver_enable|$FORM_main_webserver_enable|true|Enabled
radio|main_webserver_enable|$FORM_main_webserver_enable|false|Disabled
field|Port
text|webserver_port|$FORM_webserver_port
field|<br />
helpitem|Phidget Server
helptext|The Phidget server enables remote access to Phidgets attached to this SBC.
helpitem|Publish
helptext|Publish enabled client discovery and connection via server id. If disabled, clients must connect using IP address/Port.
helpitem|Server ID
helptext|The Server ID is used when connecting to the server by name (mDNS). By default this is the same as the machine hostname.
helpitem|Port
helptext|The port that the server will run on. Default value is 5661.
helpitem|Password
helptext|The password that the server will run with. By default this is blank, and no password will be required.
helpitem|Web Server
helptext|The web server enables access to the Phidget server from web clients via the Javascript API. Available <a href="http://$_host:$FORM_webserver_port/" target="_blank">here</a>.
helpitem|Port
helptext|The port that the web server will run on. Default value is 8080.
end_form
EOF

echo "<br><br>"

display_form <<EOF
start_form|Phidget Dictionaries
field|
radio|dictenabled|$FORM_dictenabled|true|Enabled
radio|dictenabled|$FORM_dictenabled|false|Disabled
field|<br>
field|Monitoring / Control Dictionary
radio|controldictenabled|$FORM_controldictenabled|true|Enabled
radio|controldictenabled|$FORM_controldictenabled|false|Disabled
field|<br />
EOF


[ "$_p22ns_dictdir_exists" = "true" ] && neq "$(ls -1q $_p22ns_dictdir/*.dpc | wc -l)" "0" && {

	cat <<EOF
<tr>
<td class="td40"><h4>Saved Dictionaries</h4></td><td class="td60">
</td></tr>
<tr>
<td class="td40"><br /></td><td class="td60">
</td></tr>
</tr>
<tr>
	<td colspan="2" id="mytable">
	<table>
	<tr><td></td><td align="left">Serial Number</td><td align="right">Label</td><td align="right">Status</td><td align="right"></td></tr>
EOF

	odd=true
	for file in $_p22ns_dictdir/*.dpc; do
		_dict_enabled=$(pconf_get $file dictionary.enabled 0)
		_dict_sn=$(pconf_get $file dictionary.sn)
		_dict_label=$(pconf_get $file dictionary.label)
		[ "$_dict_enabled" = "1" ] && status="<span style=\"color:green\">enabled</span>" || status="<span style=\"color:red\">disabled</span>"

		if equal "$odd" "true"; then
			tr_class="class=\"odd\""
			odd=false
		else
			tr_class=
			odd=true
		fi

		cat << EOF
<tr $tr_class><td><input type="radio" name="dict_select" value="$_dict_sn" onclick="saved_dict_selected($_dict_sn)" /><br /></td>
<td align="left">$_dict_sn</td>
<td align="right">$_dict_label</td>
<td align="right">$status</td>
<td align="right">$current_column</td></tr>
EOF
	done

	cat <<EOF
	</table>
	</td>
</tr>
<tr>
	<td colspan="2">
		<input id="deletedict" type="submit" name="deletedict" value="Delete This Dictionary" disabled="disabled" />
		<input id="enabledict" type="submit" name="enabledict" value="Enable" disabled="disabled" />
		<input id="disabledict" type="submit" name="disabledict" value="Disable" disabled="disabled" />
	</td>
</tr>
EOF

}

	cat <<EOF
<tr>
<td class="td40"><h4>Add Dictionary</h4></td><td class="td60">
</td></tr>
<tr>
<td class="td40"><br /></td><td class="td60">
</td></tr>
<tr>
<tr>
<td class="td40">Serial Number</td><td class="td60">
<input id="newserial" type="text" name="newserial" />
</td></tr>
<tr>
<td class="td40">Label</td><td class="td60">
<input id="newlabel" type="text" name="newlabel" />
</td></tr>
<td colspan="2">
<input id="newdict" type="submit" name="newdict" value="Create new dictionary" />
</td></tr>

</table>
</td>
EOF

	cat <<EOF
<td class="settings-help" valign="top">
<h4>Notes:</h4><p>Saved dictionaries are created and hosted by the network server on startup.</p>
		<h4>Monitoring / Control Dictionary:</h4><p>This is a special dictionary which clients can use to get information
		about connected clients and opened Phidgets.</p>
		<h4>Adding:</h4><p>New dictionaries can be created. Serial number is required and must be >= 1000.</p>
		<h4>Deleting:</h4><p>Delete a saved dictionary. There is no confirmation and this cannot be undone.</p>
		<h4>Enable / Disable:</h4><p>Dictionaries that are enabled will be created. Disabled dictionary are not created.</p>
  </td>
</tr>
</table>
<br><br>
EOF

else
	if [ "$installed" = "true" ]; then
		cat <<EOF
The phidget22 network server config file /etc/phidgets/phidget22networkserver.pc is missing. Reinstall the network server or recreate this file.
EOF
	else
		cat <<EOF
The phidget22 network server is not installed. You can install it from the <a href="/cgi-bin/system-packages.sh">Packages</a> page, or via SSH.
EOF
	fi
fi

footer ?>

<!--
##WEBIF:name:Phidgets:201:phidget22 Network Server
-->
